/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

describe('UserProfileEntity', function () {
  var model;

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('user-profile-entity', function () {
      done();
    });
  });

  beforeEach(function () {
    model = {
      uuid: 'env1',
      name: 'sandbox',
      lastConnection: 'today',
      owner: 'test-owner'
    };
  });

  it('can render a model', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.UserProfileEntity,
      {
        entity: model,
        switchModel: sinon.stub(),
        type: 'model' },
      React.createElement(
        'span',
        null,
        'Summary details'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    var output = renderer.getRenderOutput();
    var button = output.props.children[1].props.children.props.children[0].props.children[1].props.children;
    var expected = React.createElement(
      'li',
      { className: 'user-profile__entity user-profile__list-row twelve-col',
        onClick: instance._toggle },
      React.createElement(
        'div',
        { className: 'user-profile__entity-summary twelve-col no-margin-bottom' },
        React.createElement(
          'span',
          null,
          'Summary details'
        )
      ),
      React.createElement(
        'div',
        { className: 'user-profile__entity-details twelve-col',
          style: { height: '0px', opacity: 0 } },
        React.createElement(
          'div',
          { className: 'twelve-col no-margin-bottom',
            ref: 'inner' },
          React.createElement(
            'div',
            { className: 'user-profile__entity-details-header twelve-col' },
            React.createElement(
              'div',
              { className: 'ten-col no-margin-bottom' },
              undefined,
              "sandbox"
            ),
            React.createElement(
              'div',
              { className: 'user-profile__entity-details-header-action ' + 'two-col last-col no-margin-bottom' },
              React.createElement(juju.components.GenericButton, {
                action: button.props.action,
                title: 'Manage' })
            )
          ),
          React.createElement(
            'div',
            { className: 'user-profile__entity-details-content twelve-col ' + 'no-margin-bottom' },
            undefined,
            undefined,
            React.createElement(
              'div',
              { className: 'three-col last-col' },
              'Owner: ',
              "test-owner"
            ),
            undefined,
            undefined,
            undefined,
            undefined
          )
        )
      )
    );
    assert.deepEqual(output, expected);
  });

  it('can render a bundle', function () {
    var bundle = jsTestUtils.makeEntity(true).toEntity();
    var getDiagramURL = sinon.stub().returns('bundle.svg');
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.UserProfileEntity,
      {
        changeState: sinon.stub(),
        entity: bundle,
        getDiagramURL: getDiagramURL,
        type: 'bundle' },
      React.createElement(
        'span',
        null,
        'Summary details'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    var output = renderer.getRenderOutput();
    var button = output.props.children[1].props.children.props.children[0].props.children[1].props.children;
    var tag = output.props.children[1].props.children.props.children[1].props.children[5].props.children[1].props.children[0];
    var expected = React.createElement(
      'li',
      { className: 'user-profile__entity user-profile__list-row twelve-col',
        onClick: instance._toggle },
      React.createElement(
        'div',
        { className: 'user-profile__entity-summary twelve-col no-margin-bottom' },
        React.createElement(
          'span',
          null,
          'Summary details'
        )
      ),
      React.createElement(
        'div',
        { className: 'user-profile__entity-details twelve-col',
          style: { height: '0px', opacity: 0 } },
        React.createElement(
          'div',
          { className: 'twelve-col no-margin-bottom',
            ref: 'inner' },
          React.createElement(
            'div',
            { className: 'user-profile__entity-details-header twelve-col' },
            React.createElement(
              'div',
              { className: 'ten-col no-margin-bottom' },
              undefined,
              "django-cluster"
            ),
            React.createElement(
              'div',
              { className: 'user-profile__entity-details-header-action ' + 'two-col last-col no-margin-bottom' },
              React.createElement(juju.components.GenericButton, {
                action: button.props.action,
                title: 'View' })
            )
          ),
          React.createElement(
            'div',
            { className: 'user-profile__entity-details-content twelve-col ' + 'no-margin-bottom' },
            undefined,
            React.createElement(
              'div',
              { className: 'nine-col' },
              'Composed of:',
              React.createElement(
                'ul',
                { className: 'user-profile__entity-service-list' },
                React.createElement(
                  'li',
                  { className: 'user-profile__comma-item',
                    key: 'django-cluster-service-gunicorn' },
                  'gunicorn'
                ),
                React.createElement(
                  'li',
                  { className: 'user-profile__comma-item',
                    key: 'django-cluster-service-django' },
                  'django'
                )
              )
            ),
            React.createElement(
              'div',
              { className: 'three-col last-col' },
              'Owner: ',
              "test-owner"
            ),
            React.createElement(
              'div',
              { className: 'user-profile__entity-diagram twelve-col' },
              React.createElement('object', { type: 'image/svg+xml', data: 'bundle.svg',
                className: 'entity-content__diagram-image' })
            ),
            React.createElement(
              'div',
              { className: 'twelve-col no-margin-bottom' },
              React.createElement(
                'div',
                { className: 'two-col' },
                'Description'
              ),
              React.createElement(
                'div',
                { className: 'ten-col last-col' },
                'HA Django cluster.'
              )
            ),
            React.createElement(
              'div',
              { className: 'twelve-col no-margin-bottom' },
              React.createElement(
                'div',
                { className: 'two-col' },
                'Tags'
              ),
              React.createElement(
                'ul',
                { className: 'ten-col last-col' },
                [React.createElement(
                  'li',
                  { className: 'user-profile__comma-item link',
                    key: 'django-cluster-database',
                    onClick: tag.props.onClick,
                    role: 'button',
                    tabIndex: '0' },
                  'database'
                )]
              )
            ),
            undefined
          )
        )
      )
    );
    assert.deepEqual(output, expected);
  });

  it('can render a charm', function () {
    var charm = jsTestUtils.makeEntity().toEntity();
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.UserProfileEntity,
      {
        changeState: sinon.stub(),
        entity: charm,
        type: 'charm' },
      React.createElement(
        'span',
        null,
        'Summary details'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    var output = renderer.getRenderOutput();
    var button = output.props.children[1].props.children.props.children[0].props.children[1].props.children;
    var tag = output.props.children[1].props.children.props.children[1].props.children[5].props.children[1].props.children[0];
    var expected = React.createElement(
      'li',
      { className: 'user-profile__entity user-profile__list-row twelve-col',
        onClick: instance._toggle },
      React.createElement(
        'div',
        { className: 'user-profile__entity-summary twelve-col no-margin-bottom' },
        React.createElement(
          'span',
          null,
          'Summary details'
        )
      ),
      React.createElement(
        'div',
        { className: 'user-profile__entity-details twelve-col',
          style: { height: '0px', opacity: 0 } },
        React.createElement(
          'div',
          { className: 'twelve-col no-margin-bottom',
            ref: 'inner' },
          React.createElement(
            'div',
            { className: 'user-profile__entity-details-header twelve-col' },
            React.createElement(
              'div',
              { className: 'ten-col no-margin-bottom' },
              React.createElement('img', { className: 'user-profile__entity-icon',
                src: undefined,
                title: 'django' }),
              'django'
            ),
            React.createElement(
              'div',
              { className: 'user-profile__entity-details-header-action ' + 'two-col last-col no-margin-bottom' },
              React.createElement(juju.components.GenericButton, {
                action: button.props.action,
                title: 'View' })
            )
          ),
          React.createElement(
            'div',
            { className: 'user-profile__entity-details-content twelve-col ' + 'no-margin-bottom' },
            React.createElement(
              'div',
              { className: 'nine-col' },
              'Series: ',
              "trusty"
            ),
            undefined,
            React.createElement(
              'div',
              { className: 'three-col last-col' },
              'Owner: ',
              "test-owner"
            ),
            undefined,
            React.createElement(
              'div',
              { className: 'twelve-col no-margin-bottom' },
              React.createElement(
                'div',
                { className: 'two-col' },
                'Description'
              ),
              React.createElement(
                'div',
                { className: 'ten-col last-col' },
                'Django framework.'
              )
            ),
            React.createElement(
              'div',
              { className: 'twelve-col no-margin-bottom' },
              React.createElement(
                'div',
                { className: 'two-col' },
                'Tags'
              ),
              React.createElement(
                'ul',
                { className: 'ten-col last-col' },
                [React.createElement(
                  'li',
                  { className: 'user-profile__comma-item link',
                    key: 'cs:django-database',
                    onClick: tag.props.onClick,
                    role: 'button',
                    tabIndex: '0' },
                  'database'
                )]
              )
            ),
            undefined
          )
        )
      )
    );
    assert.deepEqual(output, expected);
  });

  it('can switch envs for a model', function () {
    var switchModel = sinon.stub();
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.UserProfileEntity,
      {
        entity: model,
        switchModel: switchModel,
        type: 'model' },
      React.createElement(
        'span',
        null,
        'Summary details'
      )
    ), true);
    var output = renderer.getRenderOutput();
    output.props.children[1].props.children.props.children[0].props.children[1].props.children.props.action();
    assert.equal(switchModel.callCount, 1);
    assert.equal(switchModel.args[0][0], 'env1');
    assert.equal(switchModel.args[0][1], 'sandbox');
  });

  it('can navigate to view a charm or bundle', function () {
    var bundle = jsTestUtils.makeEntity(true).toEntity();
    bundle.id = 'cs:django-cluster';
    var getDiagramURL = sinon.stub().returns('bundle.svg');
    var changeState = sinon.stub();
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.UserProfileEntity,
      {
        changeState: changeState,
        entity: bundle,
        getDiagramURL: getDiagramURL,
        type: 'bundle' },
      React.createElement(
        'span',
        null,
        'Summary details'
      )
    ), true);
    var output = renderer.getRenderOutput();
    output.props.children[1].props.children.props.children[0].props.children[1].props.children.props.action();
    assert.equal(changeState.callCount, 1);
    assert.deepEqual(changeState.args[0][0], {
      sectionC: {
        component: 'charmbrowser',
        metadata: {
          activeComponent: 'entity-details',
          id: 'django-cluster'
        }
      }
    });
  });

  it('can navigate to a tag search', function () {
    var bundle = jsTestUtils.makeEntity(true).toEntity();
    var getDiagramURL = sinon.stub().returns('bundle.svg');
    var changeState = sinon.stub();
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.UserProfileEntity,
      {
        changeState: changeState,
        entity: bundle,
        getDiagramURL: getDiagramURL,
        type: 'bundle' },
      React.createElement(
        'span',
        null,
        'Summary details'
      )
    ), true);
    var output = renderer.getRenderOutput();
    output.props.children[1].props.children.props.children[1].props.children[5].props.children[1].props.children[0].props.onClick();
    assert.equal(changeState.callCount, 1);
    assert.deepEqual(changeState.args[0][0], {
      sectionC: {
        component: 'charmbrowser',
        metadata: {
          activeComponent: 'search-results',
          search: null,
          tags: 'database'
        }
      }
    });
  });

  it('can toggle to the expanded view', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfileEntity, {
      entity: model,
      switchModel: sinon.stub(),
      type: 'model' }), true);
    var instance = renderer.getMountedInstance();
    var output = renderer.getRenderOutput();
    // Mock the ref.
    instance.refs = { inner: { offsetHeight: 10 } };
    output.props.onClick();
    output = renderer.getRenderOutput();
    var expected = React.createElement(
      'li',
      { className: 'user-profile__entity user-profile__list-row twelve-col ' + 'user-profile__entity--expanded',
        onClick: instance._toggle },
      output.props.children
    );
    assert.deepEqual(output, expected);
  });

  it('can initially be expanded', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfileEntity, {
      entity: model,
      expanded: true,
      switchEnv: sinon.stub(),
      type: 'model' }), true);
    var instance = renderer.getMountedInstance();
    // Mock the ref.
    instance.refs = { inner: { offsetHeight: 10 } };
    // The shallow renderer does not call componentDidMount, so call it
    // manually.
    instance.componentDidMount();
    var output = renderer.getRenderOutput();
    var expected = React.createElement(
      'li',
      { className: 'user-profile__entity user-profile__list-row twelve-col ' + 'user-profile__entity--expanded',
        onClick: instance._toggle },
      output.props.children
    );
    assert.deepEqual(output, expected);
  });
});